import 'package:flutter/material.dart';
import 'package:softcorp_flutter_core/default_card/default_card.dart';
import 'package:softcorp_flutter_core/text_theme/text_theme.dart';

import '../constants.dart';

class DefaultFormField extends StatefulWidget {
  final TextEditingController textEditingController;
  final TextInputType textInputType;
  final String hint;
  final String label;
  final String? Function(String?)? validator;
  final BoxConstraints? constraints;
  final int? numberOfLines;
  final bool? disabled;
  final bool? addDefaultBorder;
  final Color? labelColor;
  final IconData? prefixIcon;
  final bool? obscureText;

  const DefaultFormField({
    Key? key,
    required this.textEditingController,
    required this.textInputType,
    required this.hint,
    required this.label,
    this.validator,
    this.addDefaultBorder,
    this.constraints,
    this.numberOfLines,
    this.disabled,
    this.labelColor,
    this.prefixIcon,
    this.obscureText,
  }) : super(key: key);

  @override
  State<StatefulWidget> createState() {
    return _DefaultFormFieldState();
  }
}

class _DefaultFormFieldState extends State<DefaultFormField> {
  late Color shadowColor;
  late double cardElevation;

  @override
  void initState() {
    print("init _DefaultFormFieldState");
    shadowColor = defaultLightShadow;
    cardElevation = defaultElevation;
    super.initState();
  }

  setDefault() {
    setState(() {
      shadowColor = defaultLightShadow;
      cardElevation = defaultElevation;
    });
  }

  setActive() {
    setState(() {
      shadowColor = softcorpPrimaryColor;
      cardElevation = 10;
    });
  }

  setDanger() {
    setState(() {
      shadowColor = defaultDangerColor;
      cardElevation = 10;
    });
  }

  @override
  Widget build(BuildContext context) {
    Widget defaultCard(double elevation, Color shadow) => DefaultCard(
      shadowColor: shadow,
      elevation: widget.disabled == true ? 0 : elevation,
      child: TextFormField(
        enabled: widget.disabled == null ? true : (widget.disabled),
        controller: widget.textEditingController,
        obscureText: widget.obscureText ?? false,
        keyboardType: widget.textInputType,
        style: const TextStyle(color: Colors.black),
        decoration: InputDecoration(
          hintText: widget.hint,
          hoverColor: Colors.white,
          enabledBorder: const OutlineInputBorder(borderSide: BorderSide.none),
          focusedBorder: const OutlineInputBorder(borderSide: BorderSide.none),
          errorBorder: const OutlineInputBorder(borderSide: BorderSide.none),
          focusedErrorBorder:
              const OutlineInputBorder(borderSide: BorderSide.none),
          hintStyle: const TextStyle(fontSize: 16, color: Colors.grey),
          filled: true,
          prefixIcon: widget.prefixIcon != null
              ? Icon(
                  widget.prefixIcon,
                  size: 15,
                  color: defaultSubTitleColor,
                )
              : null,
          contentPadding: const EdgeInsets.all(16),
          fillColor: widget.disabled == true ? Colors.grey[200] : Colors.white,
        ),
        validator: (String? val) {
          if (widget.validator == null) {
            return null;
          }
          String? validator = widget.validator!(val);
          if (validator != null) {
            setDanger();
          }
          return validator;
        },
        maxLines: widget.obscureText == true ? 1 : widget.numberOfLines,
      ),
    );
    if(widget.disabled == false){
      return defaultCard(defaultElevation, defaultLightShadow);
    }
    return Focus(
        onFocusChange: (hasFocus) {
          if (widget.disabled == true) {
            return;
          }
          if (hasFocus == true) {
            print("get in here...");
            setActive();
          } else {
            print("get in here... 2");
            setDefault();
          }
        },
      child: defaultCard(cardElevation, shadowColor),
    );
  }
}
